﻿using System;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Polly;
using Polly.Contrib.Simmy;
using Polly.Contrib.Simmy.Behavior;
using Polly.Contrib.Simmy.Fault;
using Polly.Wrap;

namespace SimmyLatency
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            var latencyPolicy = MonkeyPolicy.InjectLatencyAsync<HttpResponseMessage>(
                TimeSpan.FromSeconds(5),
                0.5, 
                () => true);

            IAsyncPolicy<HttpResponseMessage> timeoutPolicy = Policy.TimeoutAsync<HttpResponseMessage>(2, 
                onTimeoutAsync: (ctx, timespan, task) =>
                {
                    Console.WriteLine("Timed out while waiting for response.");
                    return Task.CompletedTask;
                });

            IAsyncPolicy<HttpResponseMessage> retryPolicy = Policy
                .HandleResult<HttpResponseMessage>(r => !r.IsSuccessStatusCode)
                .Or<Exception>()
                .RetryAsync(3, onRetry: (message, retryCount) =>
                {
                    Console.WriteLine($"Retrying: {retryCount}");
                });

            IAsyncPolicy<HttpResponseMessage> latencyTimeoutAndRetryPolicy = Policy.WrapAsync(
                retryPolicy, timeoutPolicy, latencyPolicy);

            // Add the HttpClientFactory
            services.AddHttpClient("OpenBreweryDb", client =>
            {
                client.BaseAddress = new Uri("https://api.openbrewerydb.org/breweries/");
                client.DefaultRequestHeaders.Add("Accept", "application/json");
            }).AddPolicyHandler(latencyTimeoutAndRetryPolicy);

            services.AddMvc().SetCompatibilityVersion(CompatibilityVersion.Version_2_2);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseMvc();
        }
    }
}
